import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.text.DecimalFormat;

public class Animator implements Runnable, PaintListener, DisposeListener {
	private Shell shell;
	private DataReader dataReader;
	private Data data;
	private Point location;
	private Button btnStop;
	private boolean graphFlag;
	
	String[] strVarX = {"x"}, strVarY = {"y"};
	boolean[] showTrace = {false};
	String[] radius = {"$3"};
	
	private double xFrom = 0, xTo = 30, yFrom = 0, yTo = 30;
	private int dataIndex = -1;
	private Color white;
	private Label lblTime;
	private double time;
	
	public Animator(Display display, DataReader dataReader, Point location, boolean graph) {
		shell = new Shell(display);
		this.location = location;
		graphFlag = graph;
		createContents();
		this.dataReader = dataReader;
		if (graph) {
			strVarX = new String[] {"$TIME"};
			showTrace = new boolean[] {true};
		}
	}
	public Rectangle getBounds() {
		return shell.getBounds();
	}
	
	private Canvas canvas;
	private void createContents() {
		if (graphFlag) {
			shell.setText("Graph - HccVisualizer");
		} else {
			shell.setText("Animator - HccVisualizer");
		}
		shell.addDisposeListener(this);

		FormLayout l = new FormLayout();
		l.marginHeight = l.marginWidth = 10;
		shell.setLayout(l);

		lblTime = new Label(shell, SWT.BORDER);
		if (graphFlag) {
			lblTime.setVisible(false);
		}
		lblTime.setText("0");
		lblTime.setAlignment(SWT.RIGHT);
		FormData data = new FormData();
		data.left = new FormAttachment(0, 0);
		data.top = new FormAttachment(0, 0);
		data.width = 100;
		lblTime.setLayoutData(data);
		
		canvas = new Canvas(shell, SWT.BORDER);
		white = new Color(shell.getDisplay(), 255, 255, 255);
		canvas.setBackground(white);
		canvas.addPaintListener(this);

		data = new FormData();
		data.width = data.height = 300;
		data.top = new FormAttachment(lblTime, 10);
		data.bottom = new FormAttachment(100, -30);
		data.left = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		canvas.setLayoutData(data);

		Composite com = new Composite(shell, SWT.NULL);
		data = new FormData();
		data.top = new FormAttachment(100, -30);
		data.bottom = new FormAttachment(100, 0);
		data.left = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		com.setLayoutData(data);
		
		com.setLayout(new GridLayout(3, true));
		Button button = new Button(com, SWT.PUSH);
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				stop();
				join();
				animate();
			}
		});
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		button.setLayoutData(gridData);
		if (graphFlag) {
			button.setText("redraw");
		} else {
			button.setText("start");
		}
		
		btnStop = new Button(com, SWT.PUSH);
		if (graphFlag) {
			btnStop.setVisible(false);
		}
		btnStop.setText("stop");
		btnStop.setEnabled(false);
		btnStop.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				stop();
			}
		});
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		btnStop.setLayoutData(gridData);

		button = new Button(com, SWT.PUSH);
		button.setText("setting...");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				AxisDialog dlg = new AxisDialog();
				dlg.setVarX(strVarX);
				dlg.setVarY(strVarY);
				dlg.setXFrom(xFrom);
				dlg.setXTo(xTo);
				dlg.setYFrom(yFrom);
				dlg.setYTo(yTo);
				dlg.setShowTrace(showTrace);
				dlg.setRadius(radius);
				dlg.openDialog(shell);
				strVarX = dlg.getVarX();
				strVarY = dlg.getVarY();
				xFrom = dlg.getXFrom();
				xTo = dlg.getXTo();
				yFrom = dlg.getYFrom();
				yTo = dlg.getYTo();
				radius = dlg.getRadius();
				showTrace = dlg.getShowTrace();
				canvas.redraw();
			}
		});
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		button.setLayoutData(gridData);
		
		shell.pack();
		if (location != null) {
			shell.setLocation(location);
		}
	}

	public void widgetDisposed(DisposeEvent e) {
		white.dispose();
	}

	private boolean executing = false;
	private boolean flgStop = false;
	Thread thread;
	public boolean animate() {
		//ᥤ󥦥ɥexecuteƤФ줿硢
		//ɥĤƤ뤳Ȥ⤢
		if (shell.isDisposed()) {
			return false;
		}
		synchronized (this) {
			if (executing) {
				return false;
			}
			executing = true;
			flgStop = false;
		}
		if (btnStop != null) {
			btnStop.setEnabled(true);
		}
		open();
		thread = new Thread(this);
		thread.start();
		
		return true;
	}
	public void stop() {
		synchronized (this) {
			if (executing) {
				flgStop = true;
				thread.interrupt();
			} else {
				return;
			}
		}
	}
	public void clear() {
		dataIndex = -1;
	}
	public void join() {
		if (thread == null) {
			return;
		}
		try {
			thread.join();
		} catch (InterruptedException e) {
			e.printStackTrace(); //ʤ
			throw new RuntimeException("Unexpected Interruption", e);
		}
	}

	public void paintControl(PaintEvent e) {
		paint(e.gc);
	}
	private int convert(double pos, int size, double from, double to) {
		return (int)((pos - from) * size / (to - from));
	}
	private Interval getData(int dataIndex, String var) {
		if (var.startsWith("$")) {
			//üѿ
			if (var.equals("$TIME")) {
				return new Interval(data.getTime(dataIndex), data.getTime(dataIndex));
			} else {
				try {
					//
					double val = Double.parseDouble(var.substring(1));
					return new Interval(val, val);
				} catch (NumberFormatException e) {
					return Interval.UNDEFINED;
				}
			}
		} else {
			return data.get(dataIndex, var);
		}
	}
	private void paint(GC gc, int dataIndex, int varIndex, Rectangle clientRect) {
		Interval posX = getData(dataIndex, strVarX[varIndex]);
		Interval posY = getData(dataIndex, strVarY[varIndex]);
		int r = (int)(getData(dataIndex, radius[varIndex]).to);
		if (r == 0) {
			//Ⱦ0ɽʤ褦ˤ
			return;
		}
		gc.setLineWidth(r);
		if (posX == null || posY == null) {
			//ǰΤΥå
			return;
		}
		if (posX.from == posX.to && posY.from == posY.to) {
			int x = convert(posX.from, clientRect.width, xFrom, xTo);
			int y = clientRect.height - 1 - convert(posY.from, clientRect.height, yFrom, yTo);
			gc.drawOval(x, y, 1, 1);
		} else {
			int left, right, top, bottom;
			if (posX.equals(Interval.UNDEFINED)) {
				left = 0;
				right = clientRect.width;
			} else {
				left = convert(posX.from, clientRect.width, xFrom, xTo);
				right = convert(posX.to, clientRect.width, xFrom, xTo);
			}
			if (posY.equals(Interval.UNDEFINED)) {
				top = 0;
				bottom = clientRect.height;
			} else {
				top = clientRect.height - 1 - convert(posY.to, clientRect.height, yFrom, yTo);
				bottom = clientRect.height - 1 - convert(posY.from, clientRect.height, yFrom, yTo);;
			}
			gc.drawRectangle(left, top, right- left, bottom - top);
		}
	}
	private void paint(GC gc) {
		Color defaultColor = gc.getForeground();
		Color gray = shell.getDisplay().getSystemColor(SWT.COLOR_GRAY);
		if (dataIndex == -1 || data == null) {
			return;
		}
		Rectangle clientRect = canvas.getClientArea();
		//פ
		if (!graphFlag) {
			gc.setForeground(gray);
		}
		for (int i = 0; i < strVarX.length; i++) {
			if (showTrace[i]) {
				for (int j = 0; j < dataIndex; j++) {
					paint(gc, j, i, clientRect);
				}
			}
		}
		//֤߰
		gc.setForeground(defaultColor);
		for (int i = 0; i < strVarX.length; i++) {
			paint(gc, dataIndex, i, clientRect);
		}
	}
	private static DecimalFormat timeFormat = new DecimalFormat("0.00");
	private boolean fDrawing;
	public void run() {
		try {
			double timeRatio = 1;
			
			data = dataReader.getData();
			//ѤȺΤ򤷤Ƥ
			String[] varX = this.strVarX;
			String[] varY = this.strVarY;
			int i = 0;
			long realLastTime = -1;
			double lastTime = 0.0;
			dataIndex = -1;
			fDrawing = false;
MAIN_LOOP:
			while (true) {
				//λ
				if (flgStop) {
					break;
				}
				if (data.size() == i) { //ߤΥǡκǸޤǹԤä
					if (!dataReader.isExecuting()) {
						//̥åɤʤΤǡ⤦ٳǧ
						if (data.size() == i) {
							//ǸޤǹԤäΤǽλ
							break;
						}
					}
					//ǡɲäƤǽ
					continue;
				}
				time = data.getTime(i);
				for (int j = 0; j < varX.length; j++) {
					if ((!varX[j].startsWith("$") && data.get(i, varX[j]) == null) || 
						(!varY[j].startsWith("$") && data.get(i, varY[j]) == null)) {
						//ǡϤޤʤ
						try {
							Thread.sleep(1);
						} catch (InterruptedException e) {
							//λե饰ꤵ줿Τޤ޼¹ԤΥ롼פǽλ롣
						}
						continue MAIN_LOOP;
					}
				}
				
				//
				if (!graphFlag) {
					if (realLastTime >= 0) {
						try {
							long t = System.currentTimeMillis();
							long dt = (long)((time - lastTime) / timeRatio * 1000);
							long now_dt = (t - realLastTime);
							if (dt > now_dt) {
								Thread.sleep(dt - now_dt);
							}
							realLastTime = t;
						} catch (InterruptedException e) {
							//λե饰ꤵ줿Τޤ޼¹ԤΥ롼פǽλ롣
						}
					} else {
						realLastTime = System.currentTimeMillis();
					}
					lastTime = time;
				}
				//
				dataIndex = i;
				if (!fDrawing) {
					fDrawing = true;
					try {
						shell.getDisplay().asyncExec(new Runnable() {
							public void run() {
								if (!canvas.isDisposed()) {
									if (graphFlag) {
										try {
											Thread.sleep(100);
										} catch (InterruptedException e) {}
									}
									//TODO ϰϤη
									GC gc = new GC(canvas);
				
									//֥Хåե
									Display display = shell.getDisplay();
									Rectangle clientRect = canvas.getClientArea();
									Image image = new Image(display, clientRect.width, clientRect.height);
									image.setBackground(gc.getBackground());
									GC tmpgc = new GC(image);
									paint(tmpgc);
									tmpgc.dispose();
				
									gc.drawImage(image, 0, 0);
									image.dispose();
									gc.dispose();
									
									//ɽ
									lblTime.setText(timeFormat.format(time));
									
									fDrawing = false;
								}
							}
						});
					} catch (SWTException e) {
						if (e.code == SWT.ERROR_WIDGET_DISPOSED || e.code == SWT.ERROR_DEVICE_DISPOSED) {
							//ɥĤ줿Τǽλ
							break;
						}
						throw e;
					}
				}
				i++;
			}
		} finally {
			synchronized(this) {
				executing = false;
			}
			if (btnStop != null) {
				try {
					shell.getDisplay().asyncExec(new Runnable() {
						public void run() {
							if (!btnStop.isDisposed()) {
								btnStop.setEnabled(executing);
							}
						}
					});
				} catch (SWTException e) {
					if (e.code != SWT.ERROR_WIDGET_DISPOSED && e.code != SWT.ERROR_DEVICE_DISPOSED) {
						throw e;
					}
				}
			}
		}
	}
	public void open() {
		if (shell.isDisposed()) {
			shell = new Shell();
			createContents();
		}
		shell.open();
	}
}
