import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

public class HccVisualizer {
	private String initCommand;
	private Text command;
	private Shell shell;
	private Animator animator;
	private Animator graph;
	private DataReader dataReader;
	private Output output;
	
	public void open() {
		Display display = null;
		try {
			display = new Display();
			shell = new Shell(display);
			createContents();
	
			dataReader = new DataReader(shell, btnStop);
			Rectangle rect = shell.getBounds();
			Point location = new Point(rect.x + rect.width / 2 - shell.getSize().x / 2,
										rect.y + rect.height + 10);
			animator = new Animator(display, dataReader, location, false);
			
			rect = animator.getBounds();
			location = new Point(rect.x + rect.width + 10, rect.y);
			graph = new Animator(display, dataReader, location, true);
			
			output = new Output(display);
	
			shell.open();
			animator.open();
			graph.open();
			shell.setFocus();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}			
			}
		} finally {
			if (animator != null) animator.stop();
			if (dataReader != null) dataReader.stop();
			if (shell != null && !shell.isDisposed()) shell.dispose();
			if (display != null) display.dispose();
		}
	}
	private Button btnStop;

	private void createContents() {
		shell.setLocation(50, 50);
		shell.setText("Hcc Visualiser");
		GridLayout l = new GridLayout(1, true);
		l.marginHeight = 10;
		l.marginWidth = 10;
		l.verticalSpacing = 10;
		shell.setLayout(l);

		command = new Text(shell, SWT.BORDER);
		command.setText(initCommand);
		GridData data = new GridData();
		data.widthHint = 450;
		data.horizontalAlignment = GridData.FILL;
		command.setLayoutData(data);
		
		Composite com = new Composite(shell, SWT.NULL);
		data = new GridData();
		data.horizontalAlignment = GridData.CENTER;
		com.setLayoutData(data);
		l = new GridLayout(5, true);
		com.setLayout(l);
		
		Button button = new Button(com, SWT.PUSH);
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				dataReader.stop();
				animator.stop();
				graph.stop();
				dataReader.join();
				animator.join();
				graph.join();
				animator.clear();
				graph.clear();
				
				dataReader.execute(command.getText());
				animator.animate();
				graph.animate();
			}
		});
		button.setText("execute");
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		button.setLayoutData(data);
		
		btnStop = new Button(com, SWT.PUSH);
		btnStop.setText("stop");
		btnStop.setEnabled(false);
		btnStop.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				dataReader.stop();
			}
		});
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		btnStop.setLayoutData(data);
		
		button = new Button(com, SWT.PUSH);
		button.setText("animation view");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				animator.open();
			}
		});
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		button.setLayoutData(data);

		button = new Button(com, SWT.PUSH);
		button.setText("graph view");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				graph.open();
			}
		});
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		button.setLayoutData(data);

		button = new Button(com, SWT.PUSH);
		button.setText("output");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				output.setText(dataReader.getOutputString());
				output.open();
			}
		});
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		button.setLayoutData(data);

		shell.pack();
	}
	
	public static boolean verbose = false;
	public static void main(String[] args) {
		HccVisualizer v = new HccVisualizer();

		StringBuffer buf = new StringBuffer();
		int i = 0;
//		while (true) {
//			if (i < args.length && args[i].equals("-v")) {
//				verbose = true;
//				i++;
//			} else {
//				break;
//			}
//		}
		if (i == args.length) {
			v.initCommand = "hybrid";
		} else {
			buf.append(args[i++]);
			for (; i < args.length; i++) {
				buf.append(" " + args[i]);
			}
			v.initCommand = buf.toString();
		}
		v.open();
	}
}
